import { z } from 'zod';
import { readFileSync, writeFileSync, lstatSync } from 'node:fs';
import { resolve, parse } from 'node:path';
import JoyCon from 'joycon';
import { bundleRequire } from 'bundle-require';
import { glob } from 'glob';
import meow from 'meow';
import conventionalChangelogConfigSpec from 'conventional-changelog-config-spec';
import { execFile } from 'node:child_process';
import { modify, applyEdits, parse as parse$1 } from 'jsonc-parser';
import { parse as parse$2, parseDocument } from 'yaml';
import * as cheerio from 'cheerio/slim';
import semver2 from 'semver';
import conventionalRecommendedBump from 'conventional-recommended-bump';
import conventionalChangelog from 'conventional-changelog';

// src/config/schema.js
var ChangelogPresetConfigTypeSchema = z.object({
  /**
   * The type of commit message.
   * @example "feat", "fix", "chore", etc..
   */
  type: z.string().describe('The type of commit message, such as "feat", "fix", "chore".'),
  /**
   * The scope of the commit message.
   */
  scope: z.string().optional().describe("The scope of the commit message."),
  /**
   * The section of the `CHANGELOG` the commit should show up in.
   */
  section: z.string().optional().describe("The section of the `CHANGELOG` the commit should show up in."),
  /**
   * Should show in the generated changelog message?
   */
  hidden: z.boolean().optional().describe("Should show in the generated changelog message?")
});
var ChangelogPresetConfigSchema = z.object({
  /**
   * List of explicitly supported commit message types.
   */
  types: z.array(ChangelogPresetConfigTypeSchema).describe("List of explicitly supported commit message types."),
  /**
   * A URL representing a specific commit at a hash.
   * @default "{{host}}/{{owner}}/{{repository}}/commit/{{hash}}"
   */
  commitUrlFormat: z.string().describe("A URL representing a specific commit at a hash."),
  /**
   * A URL representing the comparison between two git SHAs.
   * @default "{{host}}/{{owner}}/{{repository}}/compare/{{previousTag}}...{{currentTag}}"
   */
  compareUrlFormat: z.string().describe("A URL representing the comparison between two git SHAs."),
  /**
   * A URL representing the issue format (allowing a different URL format to be swapped in
   * for Gitlab, Bitbucket, etc).
   * @default "{{host}}/{{owner}}/{{repository}}/issues/{{id}}"
   */
  issueUrlFormat: z.string().describe("A URL representing the issue format."),
  /**
   * A URL representing a user's profile on GitHub, Gitlab, etc. This URL is used
   * for substituting @eglavin with https://github.com/eglavin in commit messages.
   * @default "{{host}}/{{user}}"
   */
  userUrlFormat: z.string().describe("A URL representing a user's profile on GitHub, Gitlab, etc."),
  /**
   * A string to be used to format the auto-generated release commit message.
   * @default "chore(release): {{currentTag}}"
   */
  releaseCommitMessageFormat: z.string().describe("A string to be used to format the auto-generated release commit message."),
  /**
   * List of prefixes used to detect references to issues.
   * @default ["#"]
   */
  issuePrefixes: z.array(z.string()).describe("List of prefixes used to detect references to issues.")
});
var ForkConfigSchema = z.object({
  // Commands
  //
  /**
   * If set, Fork-Version will print the current version and exit.
   * @default false
   */
  inspectVersion: z.boolean().describe("If set, Fork-Version will print the current version and exit."),
  // Options
  //
  /**
   * List of the files to be updated.
   * @default
   * ```js
   * ["bower.json", "deno.json", "deno.jsonc", "jsr.json", "jsr.jsonc", "manifest.json", "npm-shrinkwrap.json", "package-lock.json", "package.json"]
   * ```
   */
  files: z.array(z.string()).describe("List of the files to be updated."),
  /**
   * Glob pattern to match files to be updated.
   *
   * Internally we're using [glob](https://github.com/isaacs/node-glob) to match files.
   *
   * Read more about the pattern syntax [here](https://github.com/isaacs/node-glob/tree/v10.3.12?tab=readme-ov-file#glob-primer).
   *
   * @default undefined
   * @example "*.json"
   */
  glob: z.string().optional().describe("Glob pattern to match files to be updated."),
  /**
   * The path Fork-Version will run from.
   * @default
   * ```js
   * process.cwd()
   * ```
   */
  path: z.string().describe('The path Fork-Version will run from. Defaults to "process.cwd()".'),
  /**
   * Name of the changelog file.
   * @default "CHANGELOG.md"
   */
  changelog: z.string().describe('Name of the changelog file. Defaults to "CHANGELOG.md".'),
  /**
   * The header text for the changelog.
   * @default
   * ```markdown
   * # Changelog
   *
   * All notable changes to this project will be documented in this file. See [fork-version](https://github.com/eglavin/fork-version) for commit guidelines.
   * ```
   */
  header: z.string().describe("The header text for the changelog."),
  /**
   * Specify a prefix for the created tag.
   *
   * For instance if your version tag is prefixed by "version/" instead of "v" you have to specify
   * `tagPrefix: "version/"`.
   *
   * `tagPrefix` can also be used for a monorepo environment where you might want to deploy
   * multiple package from the same repository. In this case you can specify a prefix for
   * each package:
   *
   * | Example Value            | Tag Created                   |
   * |:-------------------------|:------------------------------|
   * | ""                       | `1.2.3`                       |
   * | "version/"               | `version/1.2.3`               |
   * | "@eglavin/fork-version-" | `@eglavin/fork-version-1.2.3` |
   *
   * @example "", "version/", "@eglavin/fork-version-"
   * @default "v"
   */
  tagPrefix: z.string().describe('Specify a prefix for the created tag. Defaults to "v".'),
  /**
   * Make a pre-release with optional label if given value is a string.
   *
   * | Example Value | Produced Version |
   * |:--------------|:-----------------|
   * | true          | `1.2.3-0`        |
   * | "alpha"       | `1.2.3-alpha-0`  |
   * | "beta"        | `1.2.3-beta-0`   |
   *
   * @example true, "alpha", "beta", "rc"
   * @default undefined
   */
  preRelease: z.string().or(z.boolean()).optional().describe("Make a pre-release with optional label if given value is a string."),
  /**
   * If set, Fork-Version will use this version instead of trying to determine one.
   * @example "1.0.0"
   * @default undefined
   */
  currentVersion: z.string().optional().describe("If set, Fork-Version will use this version instead of trying to determine one."),
  /**
   * If set, Fork-Version will attempt to update to this version, instead of incrementing using "conventional-commit".
   * @example "2.0.0"
   * @default undefined
   */
  nextVersion: z.string().optional().describe(
    'If set, Fork-Version will attempt to update to this version, instead of incrementing using "conventional-commit".'
  ),
  /**
   * Release as increments the version by the specified level. Overrides the default behaviour of "conventional-commit".
   * @example "major", "minor", "patch"
   * @default undefined
   */
  releaseAs: z.union([z.literal("major"), z.literal("minor"), z.literal("patch")]).optional().describe(
    'Release as increments the version by the specified level. Overrides the default behaviour of "conventional-commit".'
  ),
  // Flags
  //
  /**
   * Don't throw an error if multiple versions are found in the given files.
   * @default true
   */
  allowMultipleVersions: z.boolean().describe("Don't throw an error if multiple versions are found in the given files."),
  /**
   * Commit all changes, not just files updated by Fork-Version.
   * @default false
   */
  commitAll: z.boolean().describe("Commit all changes, not just files updated by Fork-Version."),
  /**
   * By default the conventional-changelog spec will only add commit types of `feat` and `fix` to the generated changelog.
   * If this flag is set, all [default commit types](https://github.com/conventional-changelog/conventional-changelog-config-spec/blob/238093090c14bd7d5151eb5316e635623ce633f9/versions/2.2.0/schema.json#L18)
   * will be added to the changelog.
   * @default false
   */
  changelogAll: z.boolean().describe(
    "If this flag is set, all default commit types will be added to the changelog, not just `feat` and `fix`."
  ),
  /**
   * Output debug information.
   * @default false
   */
  debug: z.boolean().describe("Output debug information."),
  /**
   * No output will be written to disk or committed.
   * @default false
   */
  dryRun: z.boolean().describe("No output will be written to disk or committed."),
  /**
   * Run without logging to the terminal.
   * @default false
   */
  silent: z.boolean().describe("Run without logging to the terminal."),
  /**
   * If unable to find a version in the given files, fallback and attempt to use the latest git tag.
   * @default true
   */
  gitTagFallback: z.boolean().describe(
    "If unable to find a version in the given files, fallback and attempt to use the latest git tag. Defaults to true."
  ),
  /**
   * If true, git will sign the commit with the systems GPG key.
   * @see {@link https://git-scm.com/docs/git-commit#Documentation/git-commit.txt--Sltkeyidgt Git - GPG Sign Commits}
   * @default false
   */
  sign: z.boolean().describe("If true, git will sign the commit with the systems GPG key."),
  /**
   * If true, git will run user defined git hooks before committing.
   * @default false
   */
  verify: z.boolean().describe("If true, git will run user defined git hooks before committing."),
  // Skip Steps
  //
  /**
   * Skip the bump step.
   * @default false
   */
  skipBump: z.boolean().describe("Skip the bump step."),
  /**
   * Skip the changelog step.
   * @default false
   */
  skipChangelog: z.boolean().describe("Skip the changelog step."),
  /**
   * Skip the commit step.
   * @default false
   */
  skipCommit: z.boolean().describe("Skip the commit step."),
  /**
   * Skip the tag step.
   * @default false
   */
  skipTag: z.boolean().describe("Skip the tag step."),
  /**
   * Override the default "conventional-changelog-conventionalcommits" preset configuration.
   */
  changelogPresetConfig: ChangelogPresetConfigSchema.partial().describe(
    'Override the default "conventional-changelog-conventionalcommits" preset configuration.'
  ),
  /**
   * Add a suffix to the release commit message.
   * @example "[skip ci]"
   */
  releaseMessageSuffix: z.string().optional().describe("Add a suffix to the release commit message.")
});

// src/config/defaults.ts
var DEFAULT_CONFIG = {
  // Commands
  inspectVersion: false,
  // Options
  files: [
    "package.json",
    "package-lock.json",
    "npm-shrinkwrap.json",
    "jsr.json",
    "jsr.jsonc",
    "deno.json",
    "deno.jsonc",
    "manifest.json",
    // Chrome extensions
    "bower.json"
  ],
  path: process.cwd(),
  changelog: "CHANGELOG.md",
  header: `# Changelog

All notable changes to this project will be documented in this file. See [fork-version](https://github.com/eglavin/fork-version) for commit guidelines.
`,
  tagPrefix: "v",
  // Flags
  allowMultipleVersions: true,
  commitAll: false,
  changelogAll: false,
  debug: false,
  dryRun: false,
  silent: false,
  gitTagFallback: true,
  sign: false,
  verify: false,
  // Skip Steps
  skipBump: false,
  skipChangelog: false,
  skipCommit: false,
  skipTag: false,
  changelogPresetConfig: {}
};
var helperText = `Usage:
  $ fork-version [options]

Commands:
  --help                           Show this help message.
  --version                        Show the current version of Fork-Version.
  --inspect-version                If set, Fork-Version will print the current project version and exit.

Options:
  --file, -F                       List of the files to be updated. [Default: ["bower.json", "deno.json", "deno.jsonc", "jsr.json", "jsr.jsonc", "manifest.json", "npm-shrinkwrap.json", "package-lock.json", "package.json"]]
  --glob, -G                       Glob pattern to match files to be updated.
  --path, -P                       The path Fork-Version will run from. [Default: process.cwd()]
  --changelog                      Name of the changelog file. [Default: "CHANGELOG.md"]
  --header                         The header text for the changelog.
  --tag-prefix                     Specify a prefix for the created tag. [Default: "v"]
  --pre-release                    Mark this release as a pre-release.
  --pre-release-tag                Mark this release with a tagged pre-release. [Example: "alpha", "beta", "rc"]
  --current-version                If set, Fork-Version will use this version instead of trying to determine one.
  --next-version                   If set, Fork-Version will attempt to update to this version, instead of incrementing using "conventional-commit".
  --release-as                     Release as increments the version by the specified level. [Choices: "major", "minor", "patch"]

Flags:
  --allow-multiple-versions        Don't throw an error if multiple versions are found in the given files. [Default: true]
  --commit-all                     Commit all changes, not just files updated by Fork-Version.
  --changelog-all                  If this flag is set, all default commit types will be added to the changelog.
  --debug                          Output debug information.
  --dry-run                        No output will be written to disk or committed.
  --silent                         Run without logging to the terminal.
  --git-tag-fallback               If unable to find a version in the given files, fallback and attempt to use the latest git tag. [Default: true]
  --sign                           If true, git will sign the commit with the systems GPG key.
  --verify                         If true, git will run user defined git hooks before committing.

  To negate a flag you can prefix it with "no-", for example "--no-git-tag-fallback" will not fallback to the latest git tag.

Skip Steps:
  --skip-bump                      Skip the version bump step.
  --skip-changelog                 Skip updating the changelog.
  --skip-commit                    Skip committing the changes.
  --skip-tag                       Skip tagging the commit.

Conventional Changelog Overrides:
  --commit-url-format              Override the default commit URL format.
  --compare-url-format             Override the default compare URL format.
  --issue-url-format               Override the default issue URL format.
  --user-url-format                Override the default user URL format.
  --release-commit-message-format  Override the default release commit message format.
  --release-message-suffix         Add a suffix to the end of the release message.

Exit Codes:
  0: Success
  1: General Error
  3: Config File Validation Error

Examples:
  $ fork-version
    Run fork-version in the current directory with default options.

  $ fork-version --path ./packages/my-package
    Run fork-version in the "./packages/my-package" directory.

  $ fork-version --file package.json --file MyApi.csproj
    Run fork-version and update the "package.json" and "MyApi.csproj" files.

  $ fork-version --glob "*/package.json"
    Run fork-version and update all "package.json" files in subdirectories.`;
function getCliArguments() {
  return meow(helperText, {
    importMeta: import.meta,
    booleanDefault: void 0,
    helpIndent: 0,
    flags: {
      // Commands
      inspectVersion: { type: "boolean" },
      // Options
      files: { type: "string", isMultiple: true, aliases: ["file"], shortFlag: "F" },
      glob: { type: "string", shortFlag: "G" },
      path: { type: "string", shortFlag: "P" },
      changelog: { type: "string" },
      header: { type: "string" },
      tagPrefix: { type: "string" },
      preRelease: { type: "boolean" },
      preReleaseTag: { type: "string" },
      currentVersion: { type: "string" },
      nextVersion: { type: "string" },
      releaseAs: { type: "string", choices: ["major", "minor", "patch"] },
      // Flags
      allowMultipleVersions: { type: "boolean" },
      commitAll: { type: "boolean" },
      changelogAll: { type: "boolean" },
      debug: { type: "boolean" },
      dryRun: { type: "boolean" },
      silent: { type: "boolean" },
      gitTagFallback: { type: "boolean" },
      sign: { type: "boolean" },
      verify: { type: "boolean" },
      // Skip Steps
      skipBump: { type: "boolean" },
      skipChangelog: { type: "boolean" },
      skipCommit: { type: "boolean" },
      skipTag: { type: "boolean" },
      // Changelog Overrides
      commitUrlFormat: { type: "string" },
      compareUrlFormat: { type: "string" },
      issueUrlFormat: { type: "string" },
      userUrlFormat: { type: "string" },
      releaseCommitMessageFormat: { type: "string" },
      releaseMessageSuffix: { type: "string" }
    }
  });
}
function getChangelogPresetConfig(mergedConfig, cliArgumentsFlags, detectedGitHost) {
  const preset = {
    name: "conventionalcommits"
  };
  if (typeof conventionalChangelogConfigSpec.properties === "object") {
    Object.entries(conventionalChangelogConfigSpec.properties).forEach(([key, value]) => {
      if ("default" in value && value.default !== void 0) {
        if (mergedConfig?.changelogAll && key === "types") {
          const parsedTypes = z.array(ChangelogPresetConfigTypeSchema).safeParse(value.default);
          if (parsedTypes.success) {
            parsedTypes.data.forEach((type) => {
              if (!type.section) {
                delete type.hidden;
                type.section = "Other Changes";
              }
            });
            preset[key] = parsedTypes.data;
            return;
          }
        }
        preset[key] = value.default;
      }
    });
  }
  if (detectedGitHost) {
    Object.entries(detectedGitHost).forEach(([key, value]) => {
      if (value !== void 0) {
        preset[key] = value;
      }
    });
  }
  if (mergedConfig?.changelogPresetConfig && typeof mergedConfig.changelogPresetConfig === "object") {
    Object.entries(mergedConfig.changelogPresetConfig).forEach(([key, value]) => {
      if (value !== void 0) {
        preset[key] = value;
      }
    });
  }
  if (mergedConfig?.releaseMessageSuffix && !cliArgumentsFlags?.releaseMessageSuffix) {
    preset.releaseCommitMessageFormat = `${preset.releaseCommitMessageFormat} ${mergedConfig.releaseMessageSuffix}`;
  }
  if (cliArgumentsFlags?.commitUrlFormat) {
    preset.commitUrlFormat = cliArgumentsFlags.commitUrlFormat;
  }
  if (cliArgumentsFlags?.compareUrlFormat) {
    preset.compareUrlFormat = cliArgumentsFlags.compareUrlFormat;
  }
  if (cliArgumentsFlags?.issueUrlFormat) {
    preset.issueUrlFormat = cliArgumentsFlags.issueUrlFormat;
  }
  if (cliArgumentsFlags?.userUrlFormat) {
    preset.userUrlFormat = cliArgumentsFlags.userUrlFormat;
  }
  if (cliArgumentsFlags?.releaseCommitMessageFormat) {
    preset.releaseCommitMessageFormat = cliArgumentsFlags.releaseCommitMessageFormat;
  }
  if (cliArgumentsFlags?.releaseMessageSuffix) {
    preset.releaseCommitMessageFormat = `${preset.releaseCommitMessageFormat} ${cliArgumentsFlags.releaseMessageSuffix}`;
  }
  return ChangelogPresetConfigSchema.passthrough().parse(preset);
}
async function detectGitHost(cwd) {
  const remoteUrl = await new Promise((onResolve) => {
    execFile("git", ["config", "--get", "remote.origin.url"], { cwd }, (_error, stdout) => {
      onResolve(stdout ? stdout.trim() : "");
    });
  });
  if (remoteUrl.startsWith("https://") && remoteUrl.includes("@dev.azure.com/")) {
    const match = /^https:\/\/(?<atorganisation>.*?)@dev.azure.com\/(?<organisation>.*?)\/(?<project>.*?)\/_git\/(?<repository>.*?)(?:\.git)?$/.exec(
      remoteUrl
    );
    if (match?.groups) {
      const { organisation = "", project = "", repository = "" } = match.groups;
      return {
        detectedGitHost: "Azure",
        commitUrlFormat: `{{host}}/${organisation}/${project}/_git/${repository}/commit/{{hash}}`,
        compareUrlFormat: `{{host}}/${organisation}/${project}/_git/${repository}/branchCompare?baseVersion=GT{{previousTag}}&targetVersion=GT{{currentTag}}`,
        issueUrlFormat: `{{host}}/${organisation}/${project}/_workitems/edit/{{id}}`
      };
    }
  } else if (remoteUrl.startsWith("git@ssh.dev.azure.com:")) {
    const match = /^git@ssh.dev.azure.com:v\d\/(?<organisation>.*?)\/(?<project>.*?)\/(?<repository>.*?)(?:\.git)?$/.exec(
      remoteUrl
    );
    if (match?.groups) {
      const { organisation = "", project = "", repository = "" } = match.groups;
      return {
        detectedGitHost: "Azure",
        commitUrlFormat: `{{host}}/${organisation}/${project}/_git/${repository}/commit/{{hash}}`,
        compareUrlFormat: `{{host}}/${organisation}/${project}/_git/${repository}/branchCompare?baseVersion=GT{{previousTag}}&targetVersion=GT{{currentTag}}`,
        issueUrlFormat: `{{host}}/${organisation}/${project}/_workitems/edit/{{id}}`
      };
    }
  }
  return null;
}

// src/config/user-config.ts
var PACKAGE_JSON_CONFIG_KEY = "fork-version";
async function getUserConfig() {
  const cliArguments = getCliArguments();
  const cwd = cliArguments.flags.path ? resolve(cliArguments.flags.path) : process.cwd();
  const joycon = new JoyCon({
    cwd,
    packageKey: PACKAGE_JSON_CONFIG_KEY,
    stopDir: parse(cwd).root
  });
  const configFilePath = await joycon.resolve([
    "fork.config.ts",
    "fork.config.js",
    "fork.config.cjs",
    "fork.config.mjs",
    "fork.config.json",
    "package.json"
  ]);
  const configFile = await loadConfigFile(configFilePath);
  const mergedConfig = {
    ...DEFAULT_CONFIG,
    ...configFile,
    ...cliArguments.flags
  };
  let globResults = [];
  if (mergedConfig.glob) {
    globResults = await glob(mergedConfig.glob, {
      cwd,
      ignore: ["node_modules/**"],
      nodir: true
    });
  }
  const detectedGitHost = await detectGitHost(cwd);
  return {
    ...mergedConfig,
    files: getFilesList(configFile?.files, cliArguments.flags?.files, globResults),
    path: cwd,
    preRelease: (
      // Meow doesn't support multiple flags with the same name, so we need to check both.
      cliArguments.flags.preReleaseTag ?? cliArguments.flags.preRelease ?? configFile.preRelease
    ),
    changelogPresetConfig: getChangelogPresetConfig(
      mergedConfig,
      cliArguments.flags,
      detectedGitHost
    )
  };
}
async function loadConfigFile(configFilePath) {
  if (!configFilePath) {
    return {};
  }
  if (configFilePath.endsWith("json")) {
    const fileContent2 = JSON.parse(readFileSync(configFilePath).toString());
    if (configFilePath.endsWith("package.json")) {
      if (fileContent2[PACKAGE_JSON_CONFIG_KEY] && typeof fileContent2[PACKAGE_JSON_CONFIG_KEY] === "object") {
        const parsed3 = ForkConfigSchema.partial().safeParse(fileContent2[PACKAGE_JSON_CONFIG_KEY]);
        if (!parsed3.success) {
          throw new Error(`Validation error in: ${configFilePath}`, { cause: parsed3.error });
        }
        return parsed3.data;
      }
      return {};
    }
    const parsed2 = ForkConfigSchema.partial().safeParse(fileContent2);
    if (!parsed2.success) {
      throw new Error(`Validation error in: ${configFilePath}`, { cause: parsed2.error });
    }
    return parsed2.data;
  }
  const fileContent = await bundleRequire({ filepath: configFilePath });
  const parsed = ForkConfigSchema.partial().safeParse(fileContent.mod.default || fileContent.mod);
  if (!parsed.success) {
    throw new Error(`Validation error in: ${configFilePath}`, { cause: parsed.error });
  }
  return parsed.data;
}
function getFilesList(configFiles, cliFiles, globResults) {
  const listOfFiles = /* @__PURE__ */ new Set();
  if (Array.isArray(configFiles)) {
    configFiles.forEach((file) => listOfFiles.add(file));
  }
  if (Array.isArray(cliFiles)) {
    cliFiles.forEach((file) => listOfFiles.add(file));
  }
  globResults.forEach((file) => listOfFiles.add(file));
  if (listOfFiles.size) {
    return Array.from(listOfFiles);
  }
  return DEFAULT_CONFIG.files;
}
function defineConfig(config) {
  return config;
}

// src/utils/logger.ts
var Logger = class {
  constructor(config) {
    this.config = config;
    this.log = this.log.bind(this);
    this.warn = this.warn.bind(this);
    this.error = this.error.bind(this);
    this.debug = this.debug.bind(this);
    this.disableLogs = this.config.silent || this.config.inspectVersion;
  }
  disableLogs = false;
  log(...messages) {
    if (!this.disableLogs) {
      console.log(...messages);
    }
  }
  warn(...messages) {
    if (!this.disableLogs) {
      console.warn(...messages);
    }
  }
  error(...messages) {
    if (!this.disableLogs) {
      console.error(...messages);
    }
  }
  debug(...messages) {
    if (this.config.debug && !this.disableLogs) {
      console.debug(...messages);
    }
  }
};
function fileExists(filePath) {
  try {
    return lstatSync(filePath).isFile();
  } catch (_error) {
    return false;
  }
}

// src/files/json-package.ts
var JSONPackage = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
  }
  /** Options for parsing JSON and JSONC files. */
  PARSE_OPTIONS = {
    allowEmptyContent: false,
    allowTrailingComma: true,
    disallowComments: false
  };
  /**
   * Sets a new string value at the given path in a JSON or JSONC string.
   * @param jsonc the JSON or JSONC string (the contents of a file)
   * @param jsonPath path to the value to set, as an array of segments
   * @param newString string to set the value to
   * @returns the JSON or JSONC string with the value set
   */
  setStringInJsonc(jsonc, jsonPath, newString) {
    const edits = modify(jsonc, jsonPath, newString, {});
    return applyEdits(jsonc, edits);
  }
  read(fileName) {
    const filePath = resolve(this.config.path, fileName);
    if (fileExists(filePath)) {
      const fileContents = readFileSync(filePath, "utf8");
      const parseErrors = [];
      const parsedJson = parse$1(fileContents, parseErrors, this.PARSE_OPTIONS);
      if (parseErrors.length) {
        this.logger.warn(`[File Manager] Unable to parse JSON: ${fileName}`, parseErrors);
        return void 0;
      }
      if (parsedJson?.version) {
        return {
          name: fileName,
          path: filePath,
          version: parsedJson.version,
          isPrivate: typeof parsedJson?.private === "boolean" ? parsedJson.private : true
        };
      }
      this.logger.warn(`[File Manager] Unable to determine json version: ${fileName}`);
    }
  }
  write(fileState, newVersion) {
    let fileContents = readFileSync(fileState.path, "utf8");
    const parseErrors = [];
    const parsedJson = parse$1(fileContents, parseErrors, this.PARSE_OPTIONS);
    if (parseErrors.length) {
      this.logger.warn(`[File Manager] Unable to parse JSON: ${fileState.path}`, parseErrors);
      return;
    }
    fileContents = this.setStringInJsonc(fileContents, ["version"], newVersion);
    if (parsedJson?.packages?.[""]) {
      fileContents = this.setStringInJsonc(fileContents, ["packages", "", "version"], newVersion);
    }
    writeFileSync(fileState.path, fileContents, "utf8");
  }
  isSupportedFile(fileName) {
    return fileName.endsWith(".json") || fileName.endsWith(".jsonc");
  }
};
var YAMLPackage = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
  }
  /**
   * If the version is returned with a "+" symbol in the value then the version might be from a
   * flutter `pubspec.yaml` file, if so we want to retain the input builderNumber by splitting it
   * and joining it again later.
   */
  handleBuildNumber(fileVersion) {
    const [version, builderNumber] = fileVersion.split("+");
    if (/^\d+$/.test(builderNumber)) {
      return {
        version,
        builderNumber
      };
    }
    return {
      version: fileVersion
    };
  }
  read(fileName) {
    const filePath = resolve(this.config.path, fileName);
    if (fileExists(filePath)) {
      const fileContents = readFileSync(filePath, "utf-8");
      const fileVersion = parse$2(fileContents)?.version;
      if (fileVersion) {
        const parsedVersion = this.handleBuildNumber(fileVersion);
        return {
          name: fileName,
          path: filePath,
          version: parsedVersion.version || "",
          builderNumber: parsedVersion.builderNumber ?? void 0
        };
      }
    }
    this.logger.warn(`[File Manager] Unable to determine yaml version: ${fileName}`);
  }
  write(fileState, newVersion) {
    const fileContents = readFileSync(fileState.path, "utf8");
    const yamlDocument = parseDocument(fileContents);
    let newFileVersion = newVersion;
    if (fileState.builderNumber !== void 0) {
      newFileVersion += `+${fileState.builderNumber}`;
    }
    yamlDocument.set("version", newFileVersion);
    writeFileSync(fileState.path, yamlDocument.toString(), "utf8");
  }
  isSupportedFile(fileName) {
    return fileName.endsWith(".yaml") || fileName.endsWith(".yml");
  }
};
var PlainText = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
  }
  read(fileName) {
    const filePath = resolve(this.config.path, fileName);
    if (fileExists(filePath)) {
      const fileContents = readFileSync(filePath, "utf8");
      return {
        name: fileName,
        path: filePath,
        version: fileContents || ""
      };
    }
    this.logger.warn(`[File Manager] Unable to determine plain text version: ${fileName}`);
  }
  write(fileState, newVersion) {
    writeFileSync(fileState.path, newVersion, "utf8");
  }
  isSupportedFile(fileName) {
    return fileName.endsWith("version.txt");
  }
};
var MSBuildProject = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
  }
  read(fileName) {
    const filePath = resolve(this.config.path, fileName);
    if (fileExists(filePath)) {
      const fileContents = readFileSync(filePath, "utf8");
      const $ = cheerio.load(fileContents, {
        xmlMode: true,
        xml: { decodeEntities: false }
      });
      const version = $("Project > PropertyGroup > Version").text();
      if (version) {
        return {
          name: fileName,
          path: filePath,
          version
        };
      }
      this.logger.warn(`[File Manager] Unable to determine ms-build version: ${fileName}`);
    }
  }
  write(fileState, newVersion) {
    const fileContents = readFileSync(fileState.path, "utf8");
    const $ = cheerio.load(fileContents, {
      xmlMode: true,
      xml: { decodeEntities: false }
    });
    $("Project > PropertyGroup > Version").text(newVersion);
    const updatedContent = $.xml().replaceAll('"/>', '" />');
    writeFileSync(fileState.path, updatedContent, "utf8");
  }
  isSupportedFile(fileName) {
    return [".csproj", ".dbproj", ".esproj", ".fsproj", ".props", ".vbproj", ".vcxproj"].findIndex(
      (ext) => fileName.endsWith(ext)
    ) !== -1;
  }
};

// src/files/file-manager.ts
var FileManager = class {
  constructor(config, logger) {
    this.config = config;
    this.logger = logger;
    this.JSONPackage = new JSONPackage(config, logger);
    this.YAMLPackage = new YAMLPackage(config, logger);
    this.PlainText = new PlainText(config, logger);
    this.MSBuildProject = new MSBuildProject(config, logger);
  }
  JSONPackage;
  YAMLPackage;
  PlainText;
  MSBuildProject;
  /**
   * Get the state from the given file name.
   *
   * @example
   * ```ts
   * fileManager.read("package.json");
   * ```
   *
   * @returns
   * ```json
   * { "name": "package.json", "path": "/path/to/package.json", "version": "1.2.3", "isPrivate": true }
   * ```
   */
  read(fileName) {
    const _fileName = fileName.toLowerCase();
    if (this.JSONPackage.isSupportedFile(_fileName)) {
      return this.JSONPackage.read(fileName);
    }
    if (this.YAMLPackage.isSupportedFile(_fileName)) {
      return this.YAMLPackage.read(fileName);
    }
    if (this.PlainText.isSupportedFile(_fileName)) {
      return this.PlainText.read(fileName);
    }
    if (this.MSBuildProject.isSupportedFile(_fileName)) {
      return this.MSBuildProject.read(fileName);
    }
    this.logger.error(`[File Manager] Unsupported file: ${fileName}`);
  }
  /**
   * Write the new version to the given file.
   *
   * @example
   * ```ts
   * fileManager.write(
   *   { name: "package.json", path: "/path/to/package.json", version: "1.2.2" },
   *   "1.2.3"
   * );
   * ```
   */
  write(fileState, newVersion) {
    if (this.config.dryRun) {
      return;
    }
    const _fileName = fileState.name.toLowerCase();
    if (this.JSONPackage.isSupportedFile(_fileName)) {
      return this.JSONPackage.write(fileState, newVersion);
    }
    if (this.YAMLPackage.isSupportedFile(_fileName)) {
      return this.YAMLPackage.write(fileState, newVersion);
    }
    if (this.PlainText.isSupportedFile(_fileName)) {
      return this.PlainText.write(fileState, newVersion);
    }
    if (this.MSBuildProject.isSupportedFile(_fileName)) {
      return this.MSBuildProject.write(fileState, newVersion);
    }
    this.logger.error(`[File Manager] Unsupported file: ${fileState.path}`);
  }
};
function getPriority(type) {
  return ["patch", "minor", "major"].indexOf(type ?? "");
}
function getVersionType(version) {
  const parseVersion = semver2.parse(version);
  if (parseVersion?.major) {
    return "major";
  } else if (parseVersion?.minor) {
    return "minor";
  } else if (parseVersion?.patch) {
    return "patch";
  }
  return void 0;
}
function getReleaseType(releaseType, currentVersion, preReleaseTag) {
  if (!preReleaseTag) {
    return releaseType;
  }
  const currentVersionsIsPreRelease = Array.isArray(semver2.prerelease(currentVersion));
  if (currentVersionsIsPreRelease) {
    const currentReleaseType = getVersionType(currentVersion);
    if (currentReleaseType === releaseType || getPriority(currentReleaseType) > getPriority(releaseType)) {
      return "prerelease";
    }
  }
  return `pre${releaseType}`;
}

// src/process/version.ts
async function getCurrentVersion(config, logger, git, fileManager, filesToUpdate) {
  const files = [];
  const versions = /* @__PURE__ */ new Set();
  for (const file of filesToUpdate) {
    if (await git.isIgnored(file)) {
      logger.debug(`[Git Ignored] ${file}`);
      continue;
    }
    const fileState = fileManager.read(file);
    if (fileState) {
      files.push(fileState);
      if (!config.currentVersion) {
        versions.add(fileState.version);
      }
    }
  }
  if (config.currentVersion) {
    versions.add(config.currentVersion);
  }
  if (versions.size === 0 && config.gitTagFallback) {
    const version = await git.getLatestTag(config.tagPrefix);
    if (version) {
      logger.warn(`Using latest git tag as fallback`);
      versions.add(version);
    }
  }
  if (versions.size === 0) {
    throw new Error("Unable to find current version");
  } else if (versions.size > 1) {
    if (!config.allowMultipleVersions) {
      throw new Error("Found multiple versions");
    }
    logger.warn(
      `Found multiple versions (${Array.from(versions).join(", ")}), using the higher semver version`
    );
  }
  const currentVersion = semver2.rsort(Array.from(versions))[0];
  if (config.inspectVersion) {
    console.log(currentVersion);
    process.exit(0);
  }
  logger.log(`Current version: ${currentVersion}`);
  return {
    files,
    version: currentVersion
  };
}
async function getNextVersion(config, logger, currentVersion) {
  if (config.skipBump) {
    logger.warn(`Skip bump, using ${currentVersion} as the next version`);
    return {
      version: currentVersion
    };
  }
  if (config.nextVersion && semver2.valid(config.nextVersion)) {
    logger.log(`Next version: ${config.nextVersion}`);
    return {
      version: config.nextVersion
    };
  }
  const isPreMajor = semver2.lt(currentVersion, "1.0.0");
  let recommendedBump;
  if (config.releaseAs) {
    recommendedBump = {
      releaseType: config.releaseAs,
      level: -1,
      reason: "User defined"
    };
  } else {
    try {
      recommendedBump = await conventionalRecommendedBump({
        preset: {
          name: "conventionalcommits",
          ...config.changelogPresetConfig,
          preMajor: isPreMajor
        },
        path: config.path,
        tagPrefix: config.tagPrefix,
        cwd: config.path
      });
    } catch (cause) {
      throw new Error(`[conventional-recommended-bump] Unable to determine next version`, {
        cause
      });
    }
  }
  if (recommendedBump.releaseType) {
    const releaseType = getReleaseType(
      recommendedBump.releaseType,
      currentVersion,
      config.preRelease
    );
    const nextVersion = semver2.inc(
      currentVersion,
      releaseType,
      typeof config.preRelease === "string" ? config.preRelease : void 0
    ) ?? "";
    logger.log(`Next version: ${nextVersion} (${releaseType})`);
    return {
      ...recommendedBump,
      preMajor: isPreMajor,
      releaseType,
      version: nextVersion
    };
  }
  throw new Error("Unable to find next version");
}
var RELEASE_PATTERN = /(^#+ \[?[0-9]+\.[0-9]+\.[0-9]+|<a name=)/m;
function getOldReleaseContent(filePath, exists) {
  if (exists) {
    const fileContents = readFileSync(filePath, "utf-8");
    const oldContentStart = fileContents.search(RELEASE_PATTERN);
    if (oldContentStart !== -1) {
      return fileContents.substring(oldContentStart);
    }
  }
  return "";
}
function getNewReleaseContent(config, logger, nextVersion) {
  return new Promise((onResolve) => {
    let newContent = "";
    conventionalChangelog(
      {
        preset: {
          name: "conventionalcommits",
          ...config.changelogPresetConfig
        },
        tagPrefix: config.tagPrefix,
        warn: (...message) => logger.debug("[conventional-changelog] ", ...message),
        cwd: config.path
      },
      {
        version: nextVersion
      },
      {
        merges: null,
        path: config.path
      }
    ).on("error", (cause) => {
      throw new Error("[conventional-changelog] Unable to parse changes", { cause });
    }).on("data", (chunk) => {
      newContent += chunk.toString();
    }).on("end", () => {
      onResolve(newContent);
    });
  });
}
async function updateChangelog(config, logger, nextVersion) {
  if (config.skipChangelog) {
    logger.warn("Skip changelog update");
    return;
  }
  if (config.header.search(RELEASE_PATTERN) !== -1) {
    throw new Error("Header cannot contain release pattern");
  }
  const changelogPath = resolve(config.path, config.changelog);
  if (!config.dryRun && !fileExists(changelogPath)) {
    logger.log(`Creating changelog: ${changelogPath}`);
    writeFileSync(changelogPath, "\n", "utf8");
  } else {
    logger.log(`Updating changelog: ${changelogPath}`);
  }
  const oldContent = getOldReleaseContent(changelogPath, fileExists(changelogPath));
  const newContent = await getNewReleaseContent(config, logger, nextVersion);
  if (!config.dryRun && newContent) {
    writeFileSync(
      changelogPath,
      `${config.header}
${newContent}
${oldContent}
`.trim(),
      "utf8"
    );
  }
}

// src/utils/format-commit-message.ts
function formatCommitMessage(message, version) {
  if (!message) {
    message = "chore(release): {{currentTag}}";
  }
  return message.replace(new RegExp("{{currentTag}}", "g"), version);
}

// src/process/commit.ts
async function commitChanges(config, logger, git, files, nextVersion) {
  if (config.skipCommit) {
    logger.warn("Skip commit");
    return;
  }
  logger.log("Committing changes");
  const filesToCommit = [];
  if (fileExists(resolve(config.path, config.changelog))) {
    filesToCommit.push(resolve(config.path, config.changelog));
  }
  for (const file of files) {
    filesToCommit.push(file.path);
  }
  if (filesToCommit.length === 0) {
    return;
  }
  if (config.commitAll) {
    await git.add("--all");
  } else {
    await git.add(...filesToCommit);
  }
  const shouldVerify = config.verify ? void 0 : "--no-verify";
  const shouldSign = config.sign ? "--gpg-sign" : void 0;
  await git.commit(
    shouldVerify,
    shouldSign,
    "--message",
    formatCommitMessage(config.changelogPresetConfig?.releaseCommitMessageFormat, nextVersion)
  );
}

// src/process/tag.ts
async function tagChanges(config, logger, git, nextVersion) {
  if (config.skipTag) {
    logger.warn("Skip tag creation");
    return;
  }
  const tag = `${config.tagPrefix}${nextVersion}`;
  logger.log(`Creating tag: ${tag}`);
  await git.tag(
    config.sign ? "--sign" : "--annotate",
    tag,
    "--message",
    formatCommitMessage(config.changelogPresetConfig?.releaseCommitMessageFormat, nextVersion)
  );
}

export { FileManager, ForkConfigSchema, Logger, commitChanges, defineConfig, getCurrentVersion, getNextVersion, getUserConfig, tagChanges, updateChangelog };
//# sourceMappingURL=chunk-4M7BP5DG.js.map
//# sourceMappingURL=chunk-4M7BP5DG.js.map