#!/usr/bin/env node
import { getUserConfig, Logger, FileManager, getCurrentVersion, getNextVersion, updateChangelog, commitChanges, tagChanges } from './chunk-4M7BP5DG.js';
import { writeFileSync } from 'node:fs';
import { join } from 'node:path';
import { ZodError } from 'zod';
import { execFile } from 'node:child_process';
import semver from 'semver';

var Git = class {
  constructor(config) {
    this.config = config;
    this.add = this.add.bind(this);
    this.commit = this.commit.bind(this);
    this.tag = this.tag.bind(this);
    this.isIgnored = this.isIgnored.bind(this);
    this.getCurrentBranchName = this.getCurrentBranchName.bind(this);
    this.getTags = this.getTags.bind(this);
    this.getLatestTag = this.getLatestTag.bind(this);
  }
  async execGit(command, args) {
    return new Promise((onResolve, onReject) => {
      execFile(
        "git",
        [command, ...args],
        {
          cwd: this.config.path,
          maxBuffer: Infinity
        },
        (error, stdout, stderr) => {
          if (error) {
            onReject(error);
          } else {
            onResolve(stdout ? stdout : stderr);
          }
        }
      );
    });
  }
  async add(...args) {
    if (this.config.dryRun) {
      return "";
    }
    return this.execGit("add", args.filter(Boolean));
  }
  async commit(...args) {
    if (this.config.dryRun) {
      return "";
    }
    return this.execGit("commit", args.filter(Boolean));
  }
  async tag(...args) {
    if (this.config.dryRun) {
      return "";
    }
    return this.execGit("tag", args.filter(Boolean));
  }
  async isIgnored(file) {
    try {
      await this.execGit("check-ignore", ["--no-index", file]);
      return true;
    } catch (_error) {
      return false;
    }
  }
  async getCurrentBranchName() {
    return (await this.execGit("rev-parse", ["--abbrev-ref", "HEAD"])).trim();
  }
  /**
   * `getTags` returns valid semver version tags in order of the commit history.
   *
   * Using `git log` to get the commit history, we then parse the tags from the
   * commit details which is expected to be in the following format:
   * @example
   * ```txt
   * commit 3841b1d05750d42197fe958e3d8e06df378a842d (HEAD -> main, tag: 1.0.2)
   * Author: Username <username@example.com>
   * Date:   Sat Nov 9 15:00:00 2024 +0000
   *
   *     chore(release): 1.2.3
   * ```
   *
   * - [Functionality extracted from the conventional-changelog - git-semver-tags project](https://github.com/conventional-changelog/conventional-changelog/blob/fac8045242099c016f5f3905e54e02b7d466bd7b/packages/git-semver-tags/index.js)
   * - [conventional-changelog git-semver-tags MIT Licence](https://github.com/conventional-changelog/conventional-changelog/blob/fac8045242099c016f5f3905e54e02b7d466bd7b/packages/git-semver-tags/LICENSE.md)
   */
  async getTags(tagPrefix) {
    const logOutput = await this.execGit("log", ["--decorate", "--no-color", "--date-order"]);
    const TAG_REGEX = /tag:\s*(.+?)[,)]/gi;
    const tags = [];
    let match = null;
    let tag;
    let tagWithoutPrefix;
    for (const logOutputLine of logOutput.split("\n")) {
      while (match = TAG_REGEX.exec(logOutputLine)) {
        tag = match[1];
        if (tagPrefix) {
          if (tag.startsWith(tagPrefix)) {
            tagWithoutPrefix = tag.replace(tagPrefix, "");
            if (semver.valid(tagWithoutPrefix)) {
              tags.push(tag);
            }
          }
        } else if (semver.valid(tag)) {
          tags.push(tag);
        }
      }
    }
    return tags;
  }
  async getLatestTag(tagPrefix) {
    const tags = await this.getTags(tagPrefix);
    if (!tags.length) return "";
    const cleanedTags = [];
    for (const tag of tags) {
      const cleanedTag = semver.clean(tag.replace(new RegExp(`^${tagPrefix}`), ""));
      if (cleanedTag) {
        cleanedTags.push(cleanedTag);
      }
    }
    return cleanedTags.sort(semver.rcompare)[0];
  }
};

// src/cli.ts
async function runFork() {
  const startTime = Date.now();
  const config = await getUserConfig();
  const logger = new Logger(config);
  const fileManager = new FileManager(config, logger);
  const git = new Git(config);
  logger.log(`Running fork-version - ${(/* @__PURE__ */ new Date()).toUTCString()}`);
  logger.warn(config.dryRun ? "[Dry Run] No changes will be written to disk.\n" : "");
  const current = await getCurrentVersion(config, logger, git, fileManager, config.files);
  const next = await getNextVersion(config, logger, current.version);
  logger.log("Updating files: ");
  for (const outFile of current.files) {
    logger.log(`  - ${outFile.path}`);
    fileManager.write(outFile, next.version);
  }
  await updateChangelog(config, logger, next.version);
  await commitChanges(config, logger, git, current.files, next.version);
  await tagChanges(config, logger, git, next.version);
  const branchName = await git.getCurrentBranchName();
  logger.log(
    `
Run \`git push --follow-tags origin ${branchName}\` to push the changes and the tag.`
  );
  if (current.files.some((file) => file.name === "package.json" && file.isPrivate === false)) {
    const npmTag = typeof config.preRelease === "string" ? config.preRelease : "prerelease";
    logger.log(
      `${next.releaseType}`.startsWith("pre") ? `Run \`npm publish --tag ${npmTag}\` to publish the package.` : "Run `npm publish` to publish the package."
    );
  }
  logger.debug(`Completed in ${Date.now() - startTime} ms`);
  const result = {
    config,
    current,
    next
  };
  if (!config.dryRun && config.debug) {
    writeFileSync(
      join(config.path, `fork-version-${Date.now()}.debug-log.json`),
      JSON.stringify(result, null, 2)
    );
  }
  return result;
}
runFork().catch((error) => {
  if (error instanceof Error) {
    if (error.cause instanceof ZodError) {
      console.error(error.message);
      for (const err of error.cause.errors) {
        console.log(`${err.path} => ${err.message}`);
      }
      process.exit(3);
    }
    if (error.stack) {
      console.error(error.stack);
    } else {
      console.error(error.message);
    }
  } else {
    console.error(error);
  }
  process.exit(1);
});
//# sourceMappingURL=cli.js.map
//# sourceMappingURL=cli.js.map