import { z } from 'zod';
import { ReleaseType } from 'semver';

declare const ForkConfigSchema: z.ZodObject<{
    /**
     * If set, Fork-Version will print the current version and exit.
     * @default false
     */
    inspectVersion: z.ZodBoolean;
    /**
     * List of the files to be updated.
     * @default
     * ```js
     * ["bower.json", "deno.json", "deno.jsonc", "jsr.json", "jsr.jsonc", "manifest.json", "npm-shrinkwrap.json", "package-lock.json", "package.json"]
     * ```
     */
    files: z.ZodArray<z.ZodString, "many">;
    /**
     * Glob pattern to match files to be updated.
     *
     * Internally we're using [glob](https://github.com/isaacs/node-glob) to match files.
     *
     * Read more about the pattern syntax [here](https://github.com/isaacs/node-glob/tree/v10.3.12?tab=readme-ov-file#glob-primer).
     *
     * @default undefined
     * @example "*.json"
     */
    glob: z.ZodOptional<z.ZodString>;
    /**
     * The path Fork-Version will run from.
     * @default
     * ```js
     * process.cwd()
     * ```
     */
    path: z.ZodString;
    /**
     * Name of the changelog file.
     * @default "CHANGELOG.md"
     */
    changelog: z.ZodString;
    /**
     * The header text for the changelog.
     * @default
     * ```markdown
     * # Changelog
     *
     * All notable changes to this project will be documented in this file. See [fork-version](https://github.com/eglavin/fork-version) for commit guidelines.
     * ```
     */
    header: z.ZodString;
    /**
     * Specify a prefix for the created tag.
     *
     * For instance if your version tag is prefixed by "version/" instead of "v" you have to specify
     * `tagPrefix: "version/"`.
     *
     * `tagPrefix` can also be used for a monorepo environment where you might want to deploy
     * multiple package from the same repository. In this case you can specify a prefix for
     * each package:
     *
     * | Example Value            | Tag Created                   |
     * |:-------------------------|:------------------------------|
     * | ""                       | `1.2.3`                       |
     * | "version/"               | `version/1.2.3`               |
     * | "@eglavin/fork-version-" | `@eglavin/fork-version-1.2.3` |
     *
     * @example "", "version/", "@eglavin/fork-version-"
     * @default "v"
     */
    tagPrefix: z.ZodString;
    /**
     * Make a pre-release with optional label if given value is a string.
     *
     * | Example Value | Produced Version |
     * |:--------------|:-----------------|
     * | true          | `1.2.3-0`        |
     * | "alpha"       | `1.2.3-alpha-0`  |
     * | "beta"        | `1.2.3-beta-0`   |
     *
     * @example true, "alpha", "beta", "rc"
     * @default undefined
     */
    preRelease: z.ZodOptional<z.ZodUnion<[z.ZodString, z.ZodBoolean]>>;
    /**
     * If set, Fork-Version will use this version instead of trying to determine one.
     * @example "1.0.0"
     * @default undefined
     */
    currentVersion: z.ZodOptional<z.ZodString>;
    /**
     * If set, Fork-Version will attempt to update to this version, instead of incrementing using "conventional-commit".
     * @example "2.0.0"
     * @default undefined
     */
    nextVersion: z.ZodOptional<z.ZodString>;
    /**
     * Release as increments the version by the specified level. Overrides the default behaviour of "conventional-commit".
     * @example "major", "minor", "patch"
     * @default undefined
     */
    releaseAs: z.ZodOptional<z.ZodUnion<[z.ZodLiteral<"major">, z.ZodLiteral<"minor">, z.ZodLiteral<"patch">]>>;
    /**
     * Don't throw an error if multiple versions are found in the given files.
     * @default true
     */
    allowMultipleVersions: z.ZodBoolean;
    /**
     * Commit all changes, not just files updated by Fork-Version.
     * @default false
     */
    commitAll: z.ZodBoolean;
    /**
     * By default the conventional-changelog spec will only add commit types of `feat` and `fix` to the generated changelog.
     * If this flag is set, all [default commit types](https://github.com/conventional-changelog/conventional-changelog-config-spec/blob/238093090c14bd7d5151eb5316e635623ce633f9/versions/2.2.0/schema.json#L18)
     * will be added to the changelog.
     * @default false
     */
    changelogAll: z.ZodBoolean;
    /**
     * Output debug information.
     * @default false
     */
    debug: z.ZodBoolean;
    /**
     * No output will be written to disk or committed.
     * @default false
     */
    dryRun: z.ZodBoolean;
    /**
     * Run without logging to the terminal.
     * @default false
     */
    silent: z.ZodBoolean;
    /**
     * If unable to find a version in the given files, fallback and attempt to use the latest git tag.
     * @default true
     */
    gitTagFallback: z.ZodBoolean;
    /**
     * If true, git will sign the commit with the systems GPG key.
     * @see {@link https://git-scm.com/docs/git-commit#Documentation/git-commit.txt--Sltkeyidgt Git - GPG Sign Commits}
     * @default false
     */
    sign: z.ZodBoolean;
    /**
     * If true, git will run user defined git hooks before committing.
     * @default false
     */
    verify: z.ZodBoolean;
    /**
     * Skip the bump step.
     * @default false
     */
    skipBump: z.ZodBoolean;
    /**
     * Skip the changelog step.
     * @default false
     */
    skipChangelog: z.ZodBoolean;
    /**
     * Skip the commit step.
     * @default false
     */
    skipCommit: z.ZodBoolean;
    /**
     * Skip the tag step.
     * @default false
     */
    skipTag: z.ZodBoolean;
    /**
     * Override the default "conventional-changelog-conventionalcommits" preset configuration.
     */
    changelogPresetConfig: z.ZodObject<{
        types: z.ZodOptional<z.ZodArray<z.ZodObject<{
            /**
             * The type of commit message.
             * @example "feat", "fix", "chore", etc..
             */
            type: z.ZodString;
            /**
             * The scope of the commit message.
             */
            scope: z.ZodOptional<z.ZodString>;
            /**
             * The section of the `CHANGELOG` the commit should show up in.
             */
            section: z.ZodOptional<z.ZodString>;
            /**
             * Should show in the generated changelog message?
             */
            hidden: z.ZodOptional<z.ZodBoolean>;
        }, "strip", z.ZodTypeAny, {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }, {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }>, "many">>;
        commitUrlFormat: z.ZodOptional<z.ZodString>;
        compareUrlFormat: z.ZodOptional<z.ZodString>;
        issueUrlFormat: z.ZodOptional<z.ZodString>;
        userUrlFormat: z.ZodOptional<z.ZodString>;
        releaseCommitMessageFormat: z.ZodOptional<z.ZodString>;
        issuePrefixes: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
    }, "strip", z.ZodTypeAny, {
        types?: {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }[] | undefined;
        commitUrlFormat?: string | undefined;
        compareUrlFormat?: string | undefined;
        issueUrlFormat?: string | undefined;
        userUrlFormat?: string | undefined;
        releaseCommitMessageFormat?: string | undefined;
        issuePrefixes?: string[] | undefined;
    }, {
        types?: {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }[] | undefined;
        commitUrlFormat?: string | undefined;
        compareUrlFormat?: string | undefined;
        issueUrlFormat?: string | undefined;
        userUrlFormat?: string | undefined;
        releaseCommitMessageFormat?: string | undefined;
        issuePrefixes?: string[] | undefined;
    }>;
    /**
     * Add a suffix to the release commit message.
     * @example "[skip ci]"
     */
    releaseMessageSuffix: z.ZodOptional<z.ZodString>;
}, "strip", z.ZodTypeAny, {
    inspectVersion: boolean;
    path: string;
    files: string[];
    changelog: string;
    header: string;
    tagPrefix: string;
    allowMultipleVersions: boolean;
    commitAll: boolean;
    changelogAll: boolean;
    debug: boolean;
    dryRun: boolean;
    silent: boolean;
    gitTagFallback: boolean;
    sign: boolean;
    verify: boolean;
    skipBump: boolean;
    skipChangelog: boolean;
    skipCommit: boolean;
    skipTag: boolean;
    changelogPresetConfig: {
        types?: {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }[] | undefined;
        commitUrlFormat?: string | undefined;
        compareUrlFormat?: string | undefined;
        issueUrlFormat?: string | undefined;
        userUrlFormat?: string | undefined;
        releaseCommitMessageFormat?: string | undefined;
        issuePrefixes?: string[] | undefined;
    };
    glob?: string | undefined;
    preRelease?: string | boolean | undefined;
    currentVersion?: string | undefined;
    nextVersion?: string | undefined;
    releaseAs?: "major" | "minor" | "patch" | undefined;
    releaseMessageSuffix?: string | undefined;
}, {
    inspectVersion: boolean;
    path: string;
    files: string[];
    changelog: string;
    header: string;
    tagPrefix: string;
    allowMultipleVersions: boolean;
    commitAll: boolean;
    changelogAll: boolean;
    debug: boolean;
    dryRun: boolean;
    silent: boolean;
    gitTagFallback: boolean;
    sign: boolean;
    verify: boolean;
    skipBump: boolean;
    skipChangelog: boolean;
    skipCommit: boolean;
    skipTag: boolean;
    changelogPresetConfig: {
        types?: {
            type: string;
            scope?: string | undefined;
            section?: string | undefined;
            hidden?: boolean | undefined;
        }[] | undefined;
        commitUrlFormat?: string | undefined;
        compareUrlFormat?: string | undefined;
        issueUrlFormat?: string | undefined;
        userUrlFormat?: string | undefined;
        releaseCommitMessageFormat?: string | undefined;
        issuePrefixes?: string[] | undefined;
    };
    glob?: string | undefined;
    preRelease?: string | boolean | undefined;
    currentVersion?: string | undefined;
    nextVersion?: string | undefined;
    releaseAs?: "major" | "minor" | "patch" | undefined;
    releaseMessageSuffix?: string | undefined;
}>;

type ForkConfig = z.infer<typeof ForkConfigSchema>;
type Config = Partial<ForkConfig>;

declare function getUserConfig(): Promise<ForkConfig>;
declare function defineConfig(config: Config): Config;

declare class Logger {
    private config;
    disableLogs: boolean;
    constructor(config: Pick<ForkConfig, "silent" | "debug" | "inspectVersion">);
    log(...messages: any[]): void;
    warn(...messages: any[]): void;
    error(...messages: any[]): void;
    debug(...messages: any[]): void;
}

interface FileState {
    name: string;
    path: string;
    version: string;
    [other: string]: unknown;
}
interface IFileManager {
    read(fileName: string): FileState | undefined;
    write(fileState: FileState, newVersion: string): void;
    isSupportedFile(fileName: string): boolean;
}
declare class FileManager {
    private config;
    private logger;
    private JSONPackage;
    private YAMLPackage;
    private PlainText;
    private MSBuildProject;
    constructor(config: ForkConfig, logger: Logger);
    /**
     * Get the state from the given file name.
     *
     * @example
     * ```ts
     * fileManager.read("package.json");
     * ```
     *
     * @returns
     * ```json
     * { "name": "package.json", "path": "/path/to/package.json", "version": "1.2.3", "isPrivate": true }
     * ```
     */
    read(fileName: string): FileState | undefined;
    /**
     * Write the new version to the given file.
     *
     * @example
     * ```ts
     * fileManager.write(
     *   { name: "package.json", path: "/path/to/package.json", version: "1.2.2" },
     *   "1.2.3"
     * );
     * ```
     */
    write(fileState: FileState, newVersion: string): void;
}

declare class Git {
    private config;
    constructor(config: Pick<ForkConfig, "path" | "dryRun">);
    private execGit;
    add(...args: (string | undefined)[]): Promise<string>;
    commit(...args: (string | undefined)[]): Promise<string>;
    tag(...args: (string | undefined)[]): Promise<string>;
    isIgnored(file: string): Promise<boolean>;
    getCurrentBranchName(): Promise<string>;
    /**
     * `getTags` returns valid semver version tags in order of the commit history.
     *
     * Using `git log` to get the commit history, we then parse the tags from the
     * commit details which is expected to be in the following format:
     * @example
     * ```txt
     * commit 3841b1d05750d42197fe958e3d8e06df378a842d (HEAD -> main, tag: 1.0.2)
     * Author: Username <username@example.com>
     * Date:   Sat Nov 9 15:00:00 2024 +0000
     *
     *     chore(release): 1.2.3
     * ```
     *
     * - [Functionality extracted from the conventional-changelog - git-semver-tags project](https://github.com/conventional-changelog/conventional-changelog/blob/fac8045242099c016f5f3905e54e02b7d466bd7b/packages/git-semver-tags/index.js)
     * - [conventional-changelog git-semver-tags MIT Licence](https://github.com/conventional-changelog/conventional-changelog/blob/fac8045242099c016f5f3905e54e02b7d466bd7b/packages/git-semver-tags/LICENSE.md)
     */
    getTags(tagPrefix: string | undefined): Promise<string[]>;
    getLatestTag(tagPrefix: string | undefined): Promise<string>;
}

interface CurrentVersion {
    version: string;
    files: FileState[];
}
declare function getCurrentVersion(config: ForkConfig, logger: Logger, git: Git, fileManager: FileManager, filesToUpdate: string[]): Promise<CurrentVersion>;
interface NextVersion {
    version: string;
    level?: number;
    preMajor?: boolean;
    reason?: string;
    releaseType?: ReleaseType;
}
declare function getNextVersion(config: ForkConfig, logger: Logger, currentVersion: string): Promise<NextVersion>;

declare function updateChangelog(config: ForkConfig, logger: Logger, nextVersion: string): Promise<void>;

declare function commitChanges(config: ForkConfig, logger: Logger, git: Git, files: FileState[], nextVersion: string): Promise<void>;

declare function tagChanges(config: ForkConfig, logger: Logger, git: Git, nextVersion: string): Promise<void>;

export { type Config, type CurrentVersion, FileManager, type FileState, type ForkConfig, ForkConfigSchema, type IFileManager, Logger, type NextVersion, commitChanges, defineConfig, getCurrentVersion, getNextVersion, getUserConfig, tagChanges, updateChangelog };
